@@ -10,7 +10,7 @@ from collections import Counter
 from functools import cached_property
 from pathlib import Path
 
-from cleantext import clean
+from cleantext import clean, fix_bad_unicode
 
 from .dehyphen_wrapper import dehyphen_paragraph, newline_or_not
 from .doc_info import (
@@ -56,6 +56,7 @@ def extract(
         footnotes_last=experimental,
         remove_page_number=experimental,
         lang=lang,
+        fast=fast,
         **kwargs,
     )
     return e.text(), tables_csv
@@ -127,6 +128,7 @@ class Export:
         footnotes_last=True,
         ocrd=None,
         lang="multi",
+        fast=False,
     ):
         if type(input_json) is str:
             self.input_data = json.loads(Path(input_json).read_text())
@@ -157,10 +159,21 @@ class Export:
         # The same looking font is sometimes super different for OCRd PDFs. Is it a bug?
         self.consider_font_size_linebreak = False
 
+        if fast:
+            # In the fast mode, not all elments are classified via Parsr. So we may have some leftover values with None.
+            # pd3f-core only works with non-none elements so remove them here.
+
+            # FIXME: This is dirty because `fast` is also encoded in `lang`
+            self.delete_none_elements()
+
         self.info = DocumentInfo(self.input_data)
         self.fix_headers_footers()
         self.export()
 
+    def delete_none_elements(self):
+        for p in self.input_data["pages"]:
+            p["elements"] = list(filter(None, p["elements"]))
+
     def export_header_footer(self):
         headers, footers = [], []
 
@@ -334,7 +347,9 @@ class Export:
         words, fonts = [], []
         for word in line["content"]:
             if word["type"] == "word":
-                words.append(word["content"])
+                w_fixed = word["content"]
+                w_fixed = fix_bad_unicode(w_fixed).strip()
+                words.append(w_fixed)
                 fonts.append(word["font"])
         return words, fonts
 
@@ -352,13 +367,16 @@ class Export:
         for l in raw_lines:
             rl, rf = self.line_to_words(l)
 
-            # "".isalnum() => False, so only check for lenth?
-            if not self.remove_punct_paragraph or any(map(no_alphanum_char, rl)):
-                lines.append(rl)
-                font_counter.update(rf)
-            else:
-                logger.debug(f"removing {rl} because not alpha num")
+            if len(rl) == 0:
                 lines.append(None)
+            else:
+                # "".isalnum() => False, so only check for lenth?
+                if not self.remove_punct_paragraph or any(map(no_alphanum_char, rl)):
+                    lines.append(rl)
+                    font_counter.update(rf)
+                else:
+                    logger.debug(f"removing {rl} because not alpha num")
+                    lines.append(None)
 
         lines = LinesWithNone(lines, raw_lines)
 
@@ -430,6 +448,7 @@ class Export:
 
             if self.remove_hyphens:
                 lines = dehyphen_paragraph(lines, lang=self.lang)
+
             return Element(
                 "body",
                 lines,
